@@ -1,9 +1,11 @@
 import inspect
-from typing import Sequence
+from functools import lru_cache
+from typing import Sequence, Dict, Optional
 
 from black import Mode, format_str
 from sympy import simplify, cse, symbols, numbered_symbols
 
+from expr_codegen.codes import sources_to_exprs
 from expr_codegen.expr import get_current_by_prefix, get_children, replace_exprs
 from expr_codegen.model import dag_start, dag_end, dag_middle
 
@@ -15,6 +17,7 @@ class ExprTool:
         self.get_current_func_kwargs = {}
         self.exprs_dict = {}
         self.exprs_names = []
+        self.globals_ = {}
 
     def set_current(self, func, **kwargs):
         self.get_current_func = func
@@ -73,7 +76,7 @@ class ExprTool:
         exprs = exprs + list(kwargs.values())
 
         # print(exprs)
-
+        syms = [str(s) for s in syms]
         return exprs, syms
 
     def reduce(self, repl, redu):
@@ -141,6 +144,7 @@ class ExprTool:
     def all(self, exprs_src, style: str = 'polars', template_file: str = 'template.py.j2',
             replace: bool = True, regroup: bool = False, format: bool = True,
             date='date', asset='asset',
+            alias: Dict[str, str] = {},
             extra_codes: Sequence[object] = ()):
         """功能集成版，将几个功能写到一起方便使用
 
@@ -162,6 +166,8 @@ class ExprTool:
             日期字段名
         asset:str
             资产字段名
+        alias: Dict[str,str]
+            符号别名。可以变通的传入正则符号名
         extra_codes: Sequence[object]
             需要复制到模板中的额外代码
 
@@ -184,8 +190,7 @@ class ExprTool:
         exprs_ldl, G = self.dag(True)
 
         if regroup:
-            # 因为遗传算法中的表达式是单个输入，所以没有必要优化
-            exprs_ldl.optimize(back_opt=False, chain_opt=True)
+            exprs_ldl.optimize()
 
         if style == 'polars':
             from expr_codegen.polars.code import codegen
@@ -196,6 +201,7 @@ class ExprTool:
 
         codes = codegen(exprs_ldl, exprs_src, syms_dst,
                         filename=template_file, date=date, asset=asset,
+                        alias=alias,
                         extra_codes=extra_codes)
 
         if format:
@@ -203,3 +209,55 @@ class ExprTool:
             codes = format_str(codes, mode=Mode(line_length=600, magic_trailing_comma=True))
 
         return codes, G
+
+    def exec(self, codes: str, df_input):
+        """执行代码
+
+        Notes
+        -----
+        注意生成的代码已经约定输入用df_input，输出用df_output
+
+        """
+        globals_ = {'df_input': df_input}
+        exec(codes, globals_)
+        return globals_['df_output']
+
+    @lru_cache(maxsize=64)
+    def _get_codes(self, source: str, extra_codes: str, output_file: str) -> str:
+        """通过字符串生成代码， 加了缓存，多次调用不重复生成"""
+        raw, exprs_dict = sources_to_exprs(self.globals_, source, safe=False)
+
+        # 生成代码
+        codes, G = _TOOL_.all(exprs_dict, style='polars', template_file='template.py.j2',
+                              replace=True, regroup=True, format=True,
+                              date='date', asset='asset',
+                              # 复制了需要使用的函数，还复制了最原始的表达式
+                              extra_codes=(raw,
+                                           # 传入多个列的方法
+                                           extra_codes,
+                                           ))
+
+        if output_file is not None:
+            with open(output_file, 'w', encoding='utf-8') as f:
+                f.write(codes)
+
+        return codes
+
+
+_TOOL_ = ExprTool()
+
+
+def codegen_exec(globals_, code_block, df_input,
+                 extra_codes: str = r'CS_SW_L1 = pl.col(r"^sw_l1_\d+$")',
+                 output_file: Optional[str] = None):
+    """快速转换源代码并执行"""
+    _TOOL_.globals_ = globals_
+
+    if isinstance(code_block, str):
+        source = code_block
+    else:
+        source = inspect.getsource(code_block)
+
+    codes = _TOOL_._get_codes(source, extra_codes, output_file)
+
+    return _TOOL_.exec(codes, df_input)
